@@ -29,8 +29,8 @@
 #'   \code{"Inverse"}, \code{"Cochran"}, or \code{"GLMM"} can be
 #'   abbreviated.
 #' @param sm A character string indicating which summary measure
-#'   (\code{"IRR"}, \code{"IRD"} or \code{"IRSD"}) is to be used for
-#'   pooling of studies, see Details.
+#'   (\code{"IRR"}, \code{"IRD"}, \code{"IRSD"}, or \code{"VE"}) is to
+#'   be used for pooling of studies, see Details.
 #' @param incr A numerical value which is added to cell frequencies
 #'   for studies with a zero cell count, see Details.
 #' @param method.incr A character string indicating which continuity
@@ -91,10 +91,12 @@
 #' @param n.e Number of observations in experimental group (optional).
 #' @param n.c Number of observations in control group (optional).
 #' @param backtransf A logical indicating whether results for
-#'   incidence rate ratio (\code{sm = "IRR"}) should be back
+#'   incidence rate ratio (\code{sm = "IRR"}) and vaccine efficacy or
+#'   vaccine effectiveness (\code{sm = "VE"}) should be back
 #'   transformed in printouts and plots. If TRUE (default), results
-#'   will be presented as incidence rate ratios; otherwise log
-#'   incidence rate ratios will be shown.
+#'   will be presented as incidence rate ratios or vaccine efficacy /
+#'   effectiveness; otherwise log incidence rate ratios or log vaccine
+#'   rate ratios will be shown.
 #' @param irscale A numeric defining a scaling factor for printing of
 #'   incidence rate differences.
 #' @param irunit A character string specifying the time unit used to
@@ -131,7 +133,8 @@
 #'   intervals should be printed for subgroups.
 #' @param byvar Deprecated argument (replaced by 'subgroup').
 #' @param hakn Deprecated argument (replaced by 'method.random.ci').
-#' @param adhoc.hakn Deprecated argument (replaced by 'adhoc.hakn.ci').
+#' @param adhoc.hakn Deprecated argument (replaced by
+#'   'adhoc.hakn.ci').
 #' @param keepdata A logical indicating whether original data (set)
 #'   should be kept in meta object.
 #' @param warn A logical indicating whether warnings should be printed
@@ -157,7 +160,13 @@
 #' \item Incidence Rate Difference (\code{sm = "IRD"})
 #' \item Square root transformed Incidence Rate Difference (\code{sm =
 #'   "IRSD"})
+#' \item Vaccine efficacy or vaccine effectiveness (\code{sm = "VE"})
 #' }
+#'
+#' Note, log incidence rate ratio (logIRR) and log vaccine ratio
+#' (logVR) are mathematical identical, however, back-transformed
+#' results differ as vaccine efficacy or effectiveness is defined as
+#' \code{VE = 100 * (1 - IRR)}.
 #' 
 #' A three-level random effects meta-analysis model (Van den Noortgate
 #' et al., 2013) is utilized if argument \code{cluster} is used and at
@@ -285,7 +294,7 @@
 #' An object of class \code{c("metainc", "meta")} with corresponding
 #' generic functions (see \code{\link{meta-object}}).
 #' 
-#' @author Guido Schwarzer \email{sc@@imbi.uni-freiburg.de}
+#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
 #' 
 #' @seealso \code{\link{meta-package}}, \code{\link{metabin}},
 #'   \code{\link{update.meta}}, \code{\link{print.meta}}
@@ -387,7 +396,7 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
                     random = gs("random") | !is.null(tau.preset),
                     overall = common | random,
                     overall.hetstat = common | random,
-                    prediction = gs("prediction"),
+                    prediction = gs("prediction") | !missing(method.predict),
                     ##
                     method.tau =
                       ifelse(!is.na(charmatch(tolower(method), "glmm",
@@ -472,7 +481,7 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
     setmethodpredict(method.predict, missing.method.predict,
                      method.tau, missing.method.tau)
   ##
-  if (method.predict == "NNF")
+  if (any(method.predict == "NNF"))
     is.installed.package("pimeta", argument = "method.predict", value = "NNF")
   ##
   missing.adhoc.hakn.pi <- missing(adhoc.hakn.pi)
@@ -488,9 +497,9 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
   if (!is.null(text.common))
     chkchar(text.common, length = 1)
   if (!is.null(text.random))
-    chkchar(text.random, length = 1)
+    chkchar(text.random)
   if (!is.null(text.predict))
-    chkchar(text.predict, length = 1)
+    chkchar(text.predict)
   if (!is.null(text.w.common))
     chkchar(text.w.common, length = 1)
   if (!is.null(text.w.random))
@@ -853,6 +862,7 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
   sel <- switch(sm,
                 IRD = event.e == 0 | event.c == 0,
                 IRR = event.e == 0 | event.c == 0,
+                VE = event.e == 0 | event.c == 0,
                 IRSD = event.e == 0 | event.c == 0)
   ##
   ## Sparse computation
@@ -870,7 +880,7 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
   else
     incr.event <- rep(0, k.all)
   ##  
-  if (sm == "IRR") {
+  if (sm %in% c("IRR", "VE")) {
     TE <- log(((event.e + incr.event) / time.e) / ((event.c + incr.event) / time.c))
     seTE <- sqrt(1 / (event.e + incr.event) + 1 / (event.c + incr.event))
   }
@@ -912,8 +922,9 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
     }
     ##
     if (!(method.tau %in% c("REML", "ML"))) {
-      if (!missing(method.tau))
-        warning("For three-level model, argument 'method.tau' set to \"REML\".",
+      if (!missing.method.tau)
+        warning("For three-level model, argument 'method.tau' set to ",
+                "\"REML\".",
                 call. = FALSE)
       method.tau <- "REML"
     }
@@ -930,51 +941,14 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
   
   ##
   ##
-  ## (9) Additional checks for GLMM, Peto method or SSW
+  ## (9) Additional checks for GLMMs
   ##
   ##
-  if (is.glmm & sm != "IRR")
-    stop("Generalised linear mixed models only possible with ",
-         "argument 'sm = \"IRR\"'.",
-         call. = FALSE)
-  ##
-  if (is.glmm & method.tau != "ML")
-    stop("Generalised linear mixed models only possible with ",
-         "argument 'method.tau = \"ML\"'.",
-         call. = FALSE)
-  ##
-  if (is.glmm & method.random.ci == "KR")
-    stop("Kenward-Roger method for random effects meta-analysis not ",
-         "available for GLMMs.",
-         call. = FALSE)
-  ##
-  if (is.glmm & method.predict == "KR")
-    stop("Kenward-Roger method for prediction interval not ",
-         "available for GLMMs.",
-         call. = FALSE)
-  ##
-  if (is.glmm & method.predict == "NNF")
-    stop("Bootstrap method for prediction interval not ",
-         "available for GLMMs.",
-         call. = FALSE)
-  ##
-  if (is.glmm & method.random.ci == "HK" & adhoc.hakn.ci != "") {
-    if (!missing.adhoc.hakn.ci)
-      warning("Ad hoc correction for Hartung-Knapp method not ",
-              "available for GLMMs.",
-              call. = FALSE)
-    adhoc.hakn.ci <- ""
-  }
-  ##
-  if (is.glmm & method.predict == "HK" & adhoc.hakn.pi != "") {
-    if (!missing.adhoc.hakn.pi)
-      warning("Ad hoc Hartung-Knapp correction ffor prediction interval not ",
-              "available for GLMMs.",
-              call. = FALSE)
-    adhoc.hakn.pi <- ""
-  }
-  ##
   if (is.glmm) {
+    chkglmm(sm, method.tau, method.random.ci, method.predict,
+            adhoc.hakn.ci, adhoc.hakn.pi,
+            c("IRR", "VE"))
+    ##
     if (!is.null(TE.tau)) {
       if (warn)
         warning("Argument 'TE.tau' not considered for GLMM.",
@@ -988,14 +962,14 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
                 call. = FALSE)
       tau.preset <- NULL
     }
+    ##
+    if (sparse)
+      if ((!missing(incr) & any(incr != 0)) |
+          allincr | addincr)
+        warning("Note, for method = \"GLMM\", continuity correction only ",
+                "used to calculate individual study results.",
+                call. = FALSE)
   }
-  ##
-  if (is.glmm & sparse)
-    if ((!missing(incr) & any(incr != 0)) |
-        allincr | addincr)
-      warning("Note, for method = \"GLMM\", continuity correction only ",
-              "used to calculate individual study results.",
-              call. = FALSE)
   
   
   ##
@@ -1006,8 +980,9 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
   k <- sum(!is.na(event.e[!exclude]) & !is.na(event.c[!exclude]) &
            !is.na(time.e[!exclude]) & !is.na(time.c[!exclude]))
   ##
-  if (k == 1 & method.random.ci == "HK")
-    method.random.ci <- "classic"
+  for (i in seq_along(method.random.ci))
+    if (k == 1 & method.random.ci[i] == "HK")
+      method.random.ci[i] <- "classic"
   ##
   if (method == "MH") {
     ##
@@ -1021,7 +996,7 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
     N.k <- n.k + m.k
     t.k <- x.k + y.k
     ##
-    if (sm == "IRR") {
+    if (sm %in% c("IRR", "VE")) {
       D <- n.k * m.k * t.k / N.k^2
       R <- x.k * m.k / N.k
       S <- y.k * n.k / N.k
@@ -1051,7 +1026,7 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
     ## Surgeon General of the Public Health Service,
     ## Chapter 8
     ## 
-    if (sm == "IRR") {
+    if (sm %in% c("IRR", "VE")) {
       w.common <- event.c * time.e / time.c
       w.common[exclude] <- 0
       TE.common <- weighted.mean(TE, w.common)
@@ -1059,38 +1034,35 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
     }
     else if (sm == "IRD") {
       warning("Cochran method only available for ",
-              "Incidence Rate Ratio (sm = \"IRR\")",
+              "Incidence Rate Ratio (sm = \"IRR\") ",
+              "and Vaccine Efficacy / Effectiveness (sm = \"VE\")",
               call. = FALSE)
       return(NULL)
     }
   }
   else if (is.glmm) {
-    zero.all <-
-      (sum(event.e[!exclude], na.rm = TRUE) == 0 &
-       sum(event.c[!exclude], na.rm = TRUE) == 0) |
-      (!any(event.e[!exclude] != time.e[!exclude]) |
-       !any(event.c[!exclude] != time.c[!exclude]))
+    list.inc <- list(x1i = event.e[!exclude], t1i = time.e[!exclude],
+                     x2i = event.c[!exclude], t2i = time.c[!exclude],
+                     measure = "IRR", model = model.glmm)
     ##
-    if (!zero.all)
-      glmm.common <-
-        runNN(rma.glmm,
-              list(x1i = event.e[!exclude], t1i = time.e[!exclude],
-                   x2i = event.c[!exclude], t2i = time.c[!exclude],
-                   method = "FE",
-                   test = ifelse(method.random.ci == "HK", "t", "z"),
-                   level = 100 * level.ma,
-                   measure = "IRR", model = model.glmm,
-                   control = control,
-                   ...))
-    else
-      glmm.common <- list(b = NA, se = NA,
-                          QE.Wld = NA, QE.df = NA, QE.LRT = NA,
-                          tau2 = NA, se.tau2 = NA)
+    use.random <-
+      sum(!exclude) > 1 &
+      !((sum(event.e[!exclude], na.rm = TRUE) == 0 &
+         sum(event.c[!exclude], na.rm = TRUE) == 0) |
+        (!any(event.e[!exclude] != time.e[!exclude]) |
+         !any(event.c[!exclude] != time.c[!exclude])))
     ##
-    TE.common   <- as.numeric(glmm.common$b)
-    seTE.common <- as.numeric(glmm.common$se)
+    res.glmm <-
+      runGLMM(list.inc,
+              method.tau = method.tau,
+              method.random.ci = method.random.ci,
+              level = level.ma,
+              control = control, use.random = use.random)
     ##
-    w.common <- rep(NA, length(time.e))
+    TE.common   <- as.numeric(res.glmm$glmm.common$b)
+    seTE.common <- as.numeric(res.glmm$glmm.common$se)
+    ##
+    w.common <- rep(NA, length(event.e))
   }
   ##
   m <- metagen(TE, seTE, studlab,
@@ -1188,199 +1160,39 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
   res$addincr <- addincr
   ##
   if (method %in% c("MH", "Cochran", "GLMM")) {
-    ##
-    ci.f <- ci(TE.common, seTE.common, level = level.ma)
-    ##
-    res$TE.common <- TE.common
-    res$seTE.common <- seTE.common
+    res <- ci2meta(res, ci.c = ci(TE.common, seTE.common, level = level.ma))
     res$w.common <- w.common
-    res$lower.common <- ci.f$lower
-    res$upper.common <- ci.f$upper
-    res$statistic.common <- ci.f$statistic
-    res$pval.common <- ci.f$p
-    res$zval.common <- ci.f$statistic
   }
   ##
   if (is.glmm) {
-    ##
-    if (sum(!exclude) > 1 & !zero.all)
-      glmm.random <-
-        runNN(rma.glmm,
-              list(x1i = event.e[!exclude], t1i = time.e[!exclude],
-                   x2i = event.c[!exclude], t2i = time.c[!exclude],
-                   method = method.tau,
-                   test = ifelse(method.random.ci == "HK", "t", "z"),
-                   level = 100 * level.ma,
-                   measure = "IRR", model = model.glmm,
-                   control = control,
-                   ...))
-    else {
-      ##
-      ## Fallback to common effect model due to small number of studies
-      ## or zero events
-      ##
-      glmm.random <- glmm.common
-    }
-    ##
-    TE.random   <- as.numeric(glmm.random$b)
-    seTE.random <- as.numeric(glmm.random$se)
-    ##
-    ci.r <- ci(TE.random, seTE.random, level = level.ma,
-               df = if (method.random.ci == "HK") k - 1)
-    ##
-    res$w.random <- rep(NA, length(event.e))
-    ##
-    res$TE.random <- TE.random
-    res$seTE.random <- seTE.random
-    res$lower.random <- ci.r$lower
-    res$upper.random <- ci.r$upper
-    res$statistic.random <- ci.r$statistic
-    res$pval.random <- ci.r$p
-    res$zval.random <- ci.r$statistic
-    ##
-    ## Prediction interval
-    ##
-    res$upper.predict <- res$lower.predict <- res$seTE.predict <- NA
-    ##
-    tau2.calc <- if (is.na(glmm.random$tau2)) 0 else glmm.random$tau2
-    seTE.predict <- sqrt(seTE.random^2 + tau2.calc)
-    ##
-    if (method.predict == "HTS" && k >= 3)
-      ci.p <- ci(TE.random, seTE.predict, level.predict, k - 2)
-    else if (method.predict == "S")
-      ci.p <- ci(TE.random, seTE.predict, level.predict)
-    else
-      ci.p <- list(lower = NA, upper = NA)
-    ##
-    res$seTE.predict <- seTE.predict
-    res$lower.predict <- ci.p$lower
-    res$upper.predict <- ci.p$upper
-    ##
+    res <- addGLMM(res, res.glmm)
     res$model.glmm <- model.glmm
     ##
-    res$Q <- if (glmm.random$k > 1) glmm.random$QE.Wld else 0
-    res$df.Q <- glmm.random$QE.df
-    res$pval.Q <- pvalQ(res$Q, res$df.Q)
-    ##
-    res$Q.LRT <- if (glmm.random$k > 1) glmm.random$QE.LRT else 0
-    res$df.Q.LRT <- res$df.Q
-    res$pval.Q.LRT <- pvalQ(res$Q.LRT, res$df.Q.LRT)
-    ##
-    if (k > 1) {
-      res$tau <- sqrt(glmm.random$tau2)
-      res$tau2 <- glmm.random$tau2
-      res$se.tau2 <- glmm.random$se.tau2
-    }
-    else
-      res$se.tau2 <- NA
-    ##
-    res$lower.tau2 <- NA
-    res$upper.tau2 <- NA
-    ##
-    res$lower.tau <- NA
-    res$upper.tau <- NA
-    ##
-    res$method.tau.ci <- ""
-    res$sign.lower.tau <- ""
-    res$sign.upper.tau <- ""
-    ##
-    H <- calcH(res$Q, res$df.Q, level.ma)
-    res$H <- H$TE
-    res$lower.H <- H$lower
-    res$upper.H <- H$upper
-    ##
-    I2 <- isquared(res$Q, res$df.Q, level.ma)
-    res$I2 <- I2$TE
-    res$lower.I2 <- I2$lower
-    res$upper.I2 <- I2$upper
-    ##
-    res$Rb <- NA
-    res$lower.Rb <- NA
-    res$upper.Rb <- NA
-    ##
-    res$.glmm.common  <- glmm.common
-    res$.glmm.random <- glmm.random
-    res$version.metafor <- packageDescription("metafor")$Version
-    ##
     if (by) {
       n.subgroups <- length(unique(subgroup[!exclude]))
       if (n.subgroups > 1)
-        subgroup.glmm <- factor(subgroup[!exclude], bylevs(subgroup[!exclude]))
-      else
-        subgroup.glmm <- NA
+        subgroup.glmm <-
+          factor(subgroup[!exclude], bylevs(subgroup[!exclude]))
       ##
-      glmm.random.by <-
-        try(suppressWarnings(
-          runNN(rma.glmm,
-                list(x1i = event.e[!exclude],
-                     t1i = time.e[!exclude],
-                     x2i = event.c[!exclude],
-                     t2i = time.c[!exclude],
-                     mods =
-                       if (n.subgroups > 1)
-                         as.call(~ subgroup.glmm) else NULL,
-                     method = method.tau,
-                     test = ifelse(method.random.ci == "HK", "t", "z"),
-                     level = 100 * level.ma,
-                     measure = "IRR", model = model.glmm,
-                     control = control,
-                     data = data.frame(subgroup.glmm),
-                     ...))),
-          silent = TRUE)
-      ##
-      if ("try-error" %in% class(glmm.random.by))
-        if (grepl(paste0("Number of parameters to be estimated is ",
-                         "larger than the number of observations"),
-                  glmm.random.by)) {
-          glmm.random.by <-
-            suppressWarnings(
-              runNN(rma.glmm,
-                    list(x1i = event.e[!exclude],
-                         t1i = time.e[!exclude],
-                         x2i = event.c[!exclude],
-                         t2i = time.c[!exclude],
-                         mods =
-                           if (n.subgroups > 1)
-                             as.call(~ subgroup.glmm) else NULL,
-                         method = "FE",
-                         test = ifelse(method.random.ci == "HK", "t", "z"),
-                         level = 100 * level.ma,
-                         measure = "IRR", model = model.glmm,
-                         control = control,
-                         data = data.frame(subgroup.glmm),
-                         ...)))
-        }
-        else
-          stop(glmm.random.by)
-      ##
-      Q.r <- glmm.random.by$QE.Wld
-      df.Q.r <- glmm.random.by$k - glmm.random.by$p
-      ##
-      H.r  <- calcH(Q.r, df.Q.r, level.ma)
-      I2.r <- isquared(Q.r, df.Q.r, level.ma)
-      ##
-      hcc <- list(tau2.resid = glmm.random.by$tau2,
-                  lower.tau2.resid = NA,
-                  upper.tau2.resid = NA,
-                  ##
-                  tau.resid = sqrt(glmm.random.by$tau2),
-                  lower.tau.resid = NA,
-                  upper.tau.resid = NA,
-                  sign.lower.tau.resid = "",
-                  sign.upper.tau.resid = "",
-                  ##
-                  Q.resid = Q.r,
-                  df.Q.resid = df.Q.r,
-                  pval.Q.resid = pvalQ(Q.r, df.Q.r),
-                  ##
-                  H.resid = H.r$TE,
-                  lower.H.resid = H.r$lower,
-                  upper.H.resid = H.r$upper,
-                  ##
-                  I2.resid = I2.r$TE,
-                  lower.I2.resid = I2.r$lower,
-                  upper.I2.resid = I2.r$upper
-                  )
+      hcc <-
+        hccGLMM(
+          res,
+          runGLMM(list.inc,
+                  method.tau = method.tau,
+                  method.random.ci = method.random.ci,
+                  level = level.ma,
+                  data =
+                    if (n.subgroups > 1)
+                      list(data = data.frame(subgroup.glmm))
+                    else
+                      NULL,
+                  mods =
+                    if (n.subgroups > 1)
+                      as.call(~ subgroup.glmm)
+                    else
+                      NULL,
+                  control = control, use.random = use.random)$glmm.random[[1]]
+        )
     }
   }
   ##
@@ -1423,33 +1235,8 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
         res <- c(res, subgroup(res, hcc$tau.resid))
     }
     ##
-    if (tau.common && is.null(tau.preset)) {
-      res$Q.w.random <- hcc$Q.resid
-      res$df.Q.w.random <- hcc$df.Q.resid
-      res$pval.Q.w.random <- hcc$pval.Q.resid
-      ##
-      res$tau2.resid <- hcc$tau2.resid
-      res$lower.tau2.resid <- hcc$lower.tau2.resid
-      res$upper.tau2.resid <- hcc$upper.tau2.resid
-      ##
-      res$tau.resid <- hcc$tau.resid
-      res$lower.tau.resid <- hcc$lower.tau.resid
-      res$upper.tau.resid <- hcc$upper.tau.resid
-      res$sign.lower.tau.resid <- hcc$sign.lower.tau.resid
-      res$sign.upper.tau.resid <- hcc$sign.upper.tau.resid
-      ##
-      res$Q.resid <- hcc$Q.resid
-      res$df.Q.resid <- hcc$df.Q.resid
-      res$pval.Q.resid <- hcc$pval.Q.resid
-      ##
-      res$H.resid <- hcc$H.resid
-      res$lower.H.resid <- hcc$lower.H.resid
-      res$upper.H.resid <- hcc$upper.H.resid
-      ##
-      res$I2.resid <- hcc$I2.resid
-      res$lower.I2.resid <- hcc$lower.I2.resid
-      res$upper.I2.resid <- hcc$upper.I2.resid
-    }
+    if (tau.common && is.null(tau.preset))
+      res <- addHet(res, hcc, !is.glmm)
     ##
     res$n.w <- NULL
     res$event.w <- NULL
@@ -1461,25 +1248,13 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
     ##
     res$n.harmonic.mean.w <- NULL
     res$t.harmonic.mean.w <- NULL
+    ##
+    res <- setNAwithin(res, res$three.level | is.glmm)
   }
   ##
   ## Backward compatibility
   ##
-  res$TE.fixed <- res$TE.common
-  res$seTE.fixed <- res$seTE.common
-  res$w.fixed <- res$w.common
-  res$lower.fixed <- res$lower.common
-  res$upper.fixed <- res$upper.common
-  res$statistic.fixed <- res$statistic.common
-  res$pval.fixed <- res$pval.common
-  res$zval.fixed <- res$zval.common
-  ##
-  if (by) {
-    res$byvar <- subgroup
-    res$bylab <- subgroup.name
-    res$print.byvar <- print.subgroup.name
-    res$byseparator <- sep.subgroup
-  }
+  res <- backward(res)
   ##
   class(res) <- c(fun, "meta")
   
