@@ -73,9 +73,7 @@
 #'   were truncated from the printout.
 #' @param details.methods A logical specifying whether details on
 #'   statistical methods should be printed.
-#' @param warn.backtransf A logical indicating whether a warning
-#'   should be printed if backtransformed proportions and rates are
-#'   below 0 and backtransformed proportions are above 1.
+#' @param warn.backtransf Deprecated argument (ignored).
 #' @param \dots Additional arguments (passed on to
 #'   \code{\link{print.meta}} called internally).
 #' 
@@ -96,7 +94,7 @@
 #' (default: "person-years"). This information is printed in summaries
 #' and forest plots if argument \code{irscale} is not equal to 1.
 #'
-#' @author Guido Schwarzer \email{sc@@imbi.uni-freiburg.de}
+#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
 #' 
 #' @seealso \code{\link{summary.meta}}, \code{\link{update.meta}},
 #'   \code{\link{metabin}}, \code{\link{metacont}},
@@ -222,8 +220,6 @@ print.summary.meta <- function(x,
   ##
   chklogical(details.methods)
   ##
-  chklogical(warn.backtransf)
-  ##
   if (!is.null(pscale))
     chknumeric(pscale, length = 1)
   else
@@ -358,9 +354,12 @@ print.summary.meta <- function(x,
         sm.lab <- "events"
     }
   }
-  else
+  else {
     if (is.relative.effect(sm))
       sm.lab <- paste0("log", sm)
+    else if (sm == "VE")
+      sm.lab <- "logVR"
+  }
   ##
   if (is.null(x$text.w.common))
     text.w.common <- paste0("%W(", gs("text.w.common"), ")")
@@ -573,7 +572,6 @@ print.summary.meta <- function(x,
                irscale = irscale, irunit = irunit, big.mark = big.mark,
                text.tau2 = text.tau2, text.tau = text.tau, text.I2 = text.I2,
                details.methods = details.methods,
-               warn.backtransf = warn.backtransf,
                ...)
   }
   else {
@@ -616,9 +614,9 @@ print.summary.meta <- function(x,
       else if (inherits(x, "metarate"))
         TE <- x$event / x$time
       else {
-        TE    <- backtransf(   TE, sm, "mean",  harmonic.mean, warn.backtransf)
-        lowTE <- backtransf(lowTE, sm, "lower", harmonic.mean, warn.backtransf)
-        uppTE <- backtransf(uppTE, sm, "upper", harmonic.mean, warn.backtransf)
+        TE    <- backtransf(   TE, sm, "mean",  harmonic.mean)
+        lowTE <- backtransf(lowTE, sm, "lower", harmonic.mean)
+        uppTE <- backtransf(uppTE, sm, "upper", harmonic.mean)
       }
       ##
       if (is.prop(sm) | sm == "RD") {
@@ -632,6 +630,12 @@ print.summary.meta <- function(x,
         lowTE <- irscale * lowTE
         uppTE <- irscale * uppTE
       }
+      ##
+      if (sm == "VE") {
+        tmp.l <- lowTE
+        lowTE <- uppTE
+        uppTE <- tmp.l
+      }
     }
     ##
     TE <- round(TE, digits)
@@ -919,7 +923,6 @@ print.summary.meta <- function(x,
                  digits.I2 = digits.I2, big.mark = big.mark,
                  text.tau2 = text.tau2, text.tau = text.tau, text.I2 = text.I2,
                  details.methods = details.methods,
-                 warn.backtransf = warn.backtransf,
                  warn.deprecated = FALSE,
                  ...)
     }
